__all__ = ['DaqChannel']
import json, logging
from dataclasses import dataclass, field
from typing import List
from dataclasses_json import dataclass_json
from ..lasp_common import Qty, SIQtys


@dataclass_json
@dataclass(eq=False)
class DaqChannel:
    channel_enabled: bool
    channel_name: str = 'Unnamed channel'
    sensitivity: float = 1.0
    range_index: int = 0
    ACCoupling_enabled: bool = False
    IEPE_enabled: bool = False
    channel_metadata: str = ''

    def __post_init__(self):
        # logging.debug(f'__post_init__({self.channel_name})')
        self._qty = SIQtys.default()
        if len(self.channel_metadata) > 0:
            meta = json.loads(self.channel_metadata)
            if 'qty' in meta:
                self._qty = Qty.from_json(meta['qty'])

    @property
    def qty(self):
        return self._qty

    @qty.setter
    def qty(self, newqty):
        self._qty = newqty
        self._store('qty', newqty)

    def _store(self, name, val):
        if len(self.channel_metadata) > 0:
            meta = json.loads(self.channel_metadata)
        else:
            meta = {}

        meta[name] = val.to_json()
        self.channel_metadata = json.dumps(meta)

    def __eq__(self, other):
        """
        We overwrite the default equal-check as the floating point values
        cannot be exactly checked due to conversion from/to JSON
        """
        return (self.channel_enabled == other.channel_enabled and
                self.channel_name == other.channel_name and 
                self.range_index == other.range_index and 
                self.ACCoupling_enabled == other.ACCoupling_enabled and 
                self.IEPE_enabled == other.IEPE_enabled and 
                self.channel_metadata == other.channel_metadata)

